from typing import Iterable, List, Dict, AnyStr, Union, Iterator, Tuple

try:
    from Quartz import CGWindowListCopyWindowInfo, kCGWindowListExcludeDesktopElements, kCGNullWindowID, \
        kCGWindowNumber, kCGWindowName, kCGWindowOwnerName, kCGWindowListOptionAll, kCGWindowListOptionOnScreenOnly, \
        kCGWindowListOptionOnScreenAboveWindow, kCGWindowListOptionOnScreenBelowWindow, \
        kCGWindowListOptionIncludingWindow

except ImportError as ex:
    raise ImportError("Please install pyobjc-framework-Quartz via pip and then try again.") from ex


WindowInfo = Dict[AnyStr, Union[AnyStr, int]]

WINDOW_OPTIONS = {'all_windows': kCGWindowListOptionAll,
                  'on_screen_only': kCGWindowListOptionOnScreenOnly,
                  'above_window': kCGWindowListOptionOnScreenAboveWindow,
                  'below_window': kCGWindowListOptionOnScreenBelowWindow,
                  'include_window': kCGWindowListOptionIncludingWindow,
                  'exclude_desktop': kCGWindowListExcludeDesktopElements}

USER_OPTS_STR = 'exclude_desktop on_screen_only'


def build_option_bitmask(*opts: Iterable[str]) -> int:
    return sum(WINDOW_OPTIONS[opt] for opt in opts if opt in WINDOW_OPTIONS)


USER_OPTIONS = build_option_bitmask(*USER_OPTS_STR.split(' '))


def get_window_info(options: int = USER_OPTIONS,
                    relative_to: int = kCGNullWindowID) -> List[WindowInfo]:
    return CGWindowListCopyWindowInfo(options, relative_to)


def gen_ids_from_info(windows: Iterable[WindowInfo]) -> Iterator[Tuple[int, str, str]]:
    for win_dict in windows:
        owner = win_dict.get('kCGWindowOwnerName', '')
        name = win_dict.get('kCGWindowName', '')
        num = win_dict.get('kCGWindowNumber', '')

        yield num, owner, name


def print_window_ids(windows):
    for info in windows:
        print(*info)


def gen_window_ids(parent: str, title: str = '', options: str = USER_OPTS_STR,
                   relative_to: bool = kCGNullWindowID) -> Iterator[int]:
    options = build_option_bitmask(*options.split(' '))
    windows = get_window_info(options, relative_to)
    parent, title = parent.lower(), title.lower()

    for num, owner, name in gen_ids_from_info(windows):
        if parent in owner.lower():
            if title:
                if title in name.lower():
                    yield num
            else:
                yield num



import os
from os import getcwd

from setuptools import setup
from pathlib import Path

# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as file:
    readme = file.read()


requirements = [line
                for line in Path('requirements.txt').read_text().split('\n')
                if line]

CMD = 'screenshot'

# OPTIONS = {
#     'argv_emulation': True,
#     'plist': {
#         'LSUIElement': True,
#     },
#     'packages': ['objc', 'cv2', 'face_recognition'],
#     'bdist_base': str(Path(getcwd()).parent) + '/build',
#     'dist_dir': str(Path(getcwd()).parent) + '/dist'
# }

setup(
    name="screenshot",
    version="1.0.0",
    author="Alex DeLorenzo",
    author_email="alexdelorenzo@gmail.com",
    description="Take screenshots on macOS",
    license="AGPL-3.0",
    keywords="screenshot macos osx apple screencapture pyscreencapture",
    url="https://github.com/thismachinechills/pyscreencapture",
    packages=['screenshot'],
    long_description=readme,
    zip_safe=True,
    install_requires=requirements,
    include_package_data=True,
    entry_points={
        'console_scripts':
            [f'{CMD} = screenshot.screencapture:run']
    },
    #
    # ## app nonsense
    # app=['brightness/change_brightness.py'],
    # datafiles=['brightness/.'],
    # WINDOW_OPTIONS={'py2app': OPTIONS},
    # setup_requires=['py2app']

)



#!/usr/bin/python3
from itertools import chain
from datetime import datetime
from subprocess import getstatusoutput
from typing import List, Iterable, Iterator

import click

from .get_window_id import gen_window_ids, WINDOW_OPTIONS, USER_OPTS_STR

FILE_EXT = '.png'
COMMAND = 'screencapture {options} -l {window} "{filename}"'
STATUS_OK = 0
STATUS_FAIL = 1

IMG_TYPES = 'png', 'pdf', 'jpg', 'tiff'


class ScreencaptureEx(Exception):
    pass


def take_screenshot(window: int, filename: str, options: List[str] = None) -> str:
    if options is None:
        options = []

    for option in options:
        if '-t' in option and not any(img_type in option.lower()
                                      for img_type in IMG_TYPES):
                raise ScreencaptureEx(f"Bad option {option}. File type unknown.")

    options = ' '.join(options)

    rc, output = getstatusoutput(COMMAND.format(window=window, filename=filename, options=options))

    if rc != STATUS_OK:
        raise ScreencaptureEx(f"Error: screencapture output: {output}")

    return filename


def get_filename(*args) -> str:
    return ' '.join(map(str, args + (datetime.now(),))) + FILE_EXT


def gen_windows(application_name: str, title: str, window_selection_options: str) -> Iterator[int]:
    windows = gen_window_ids(application_name, title, window_selection_options)

    try:
        yield next(windows)

    except StopIteration as e:
        raise ScreencaptureEx(f"Window with parent {application_name} and title {title} not found.") from e

    yield from windows


def screenshot_windows(application_name: str,
                       title: str = '',
                       window_selection_options: str = '',
                       options: List[str] = None) -> Iterator[str]:
    windows = gen_windows(application_name, title, window_selection_options)

    for window in windows:
        yield take_screenshot(window, get_filename(application_name, title), options)


def screenshot_window(application_name: str,
                      title: str = '',
                      filename: str = '',
                      window_selection_options: str = '',
                      options: List[str] = None) -> str:
    windows = gen_windows(application_name, title, window_selection_options)
    window = next(windows)
    filename = filename if filename else get_filename(application_name, title)
    return take_screenshot(window, filename, options)


@click.command()
@click.option('-w', '--window_selection_options', default=USER_OPTS_STR,
              help="Options: " + ', '.join(WINDOW_OPTIONS) + '\nDefault: ' + USER_OPTS_STR)
@click.option('-t', '--title', default='', help="Title of window from APPLICATION_NAME to capture.")
@click.option('-f', '--filename', default=None, help="Filename to save the captured PNG as.")
@click.option('-a', '--all_windows', is_flag=True, default=False, help="Capture all windows matching parameters.")
@click.option('-o', '--output', default='png',
              help="Image format to create, default is png (other options include pdf, jpg, tiff)")
@click.option('-s', '--shadow', is_flag=True, help="Capture the shadow of the window.")
@click.argument('application_name')
def run(application_name: str,
        title: str,
        filename: str,
        window_selection_options: str,
        output: str,
        shadow: bool,
        all_windows: bool):
    options = []

    if output:
        options.append(f'-t {output}')

    if not shadow:
        options.append('-o')

    try:
        if all_windows:
            if filename:
                print(f'Taking screenshots of all windows beloning to {application_name}, filename option ignored.')

            for filename in screenshot_windows(application_name, title, window_selection_options):
                print(filename)

        else:
            print(screenshot_window(application_name, title, filename, window_selection_options, options))

        exit(STATUS_OK)

    except ScreencaptureEx as e:
        print('Error:', e)
        exit(STATUS_FAIL)


if __name__ == "__main__":
    screenshot_window()
